/* regdef.h -- define register names.  */

/* This is a standard include file for MIPS targets.  Other target
   probably don't define it, and attempts to include this file will
   fail.  */

#include <machine/regdef.h>
